
#include "pesqraw.h"
// #include <stdio.h>
// #include <math.h>
// #include "pesq.h"
#include "dsp.h"


static long WB_InIIR_Nsos_8k = 1L;
static float WB_InIIR_Hsos_8k[LINIIR] = {
    2.6657628f,  -5.3315255f,  2.6657628f,  -1.8890331f,  0.89487434f };

static long WB_InIIR_Nsos_16k = 1L;
static float WB_InIIR_Hsos_16k[LINIIR] = {
    2.740826f,  -5.4816519f,  2.740826f,  -1.9444777f,  0.94597794f };

static double align_filter_dB [26] [2] = {{0.,-500},
                                 {50., -500},
                                 {100., -500},
                                 {125., -500},
                                 {160., -500},
                                 {200., -500},
                                 {250., -500},
                                 {300., -500},
                                 {350.,  0},
                                 {400.,  0},
                                 {500.,  0},
                                 {600.,  0},
                                 {630.,  0},
                                 {800.,  0},
                                 {1000., 0},
                                 {1250., 0},
                                 {1600., 0},
                                 {2000., 0},
                                 {2500., 0},
                                 {3000., 0},
                                 {3250., 0},
                                 {3500., -500},
                                 {4000., -500},
                                 {5000., -500},
                                 {6300., -500},
                                 {8000., -500}}; 


static double standard_IRS_filter_dB [26] [2] = {{  0., -200},
                                         { 50., -40}, 
                                         {100., -20},
                                         {125., -12},
                                         {160.,  -6},
                                         {200.,   0},
                                         {250.,   4},
                                         {300.,   6},
                                         {350.,   8},
                                         {400.,  10},
                                         {500.,  11},
                                         {600.,  12},
                                         {700.,  12},
                                         {800.,  12},
                                         {1000., 12},
                                         {1300., 12},
                                         {1600., 12},
                                         {2000., 12},
                                         {2500., 12},
                                         {3000., 12},
                                         {3250., 12},
                                         {3500., 4},
                                         {4000., -200},
                                         {5000., -200},
                                         {6300., -200},
                                         {8000., -200}}; 


#define TARGET_AVG_POWER    1E7

static void fix_power_level (SIGNAL_INFO *info, char *name, long maxNsamples) 
{
    long   n = info-> Nsamples;
    long   i;
    float *align_filtered = (float *) safe_malloc ((n + DATAPADDING_MSECS  * (Fs / 1000)) * sizeof (float));    
    float  global_scale;
    float  power_above_300Hz;

    for (i = 0; i < n + DATAPADDING_MSECS  * (Fs / 1000); i++) {
        align_filtered [i] = info-> data [i];
    }
    apply_filter (align_filtered, info-> Nsamples, 26, align_filter_dB);

    power_above_300Hz = (float) pow_of (align_filtered, 
                                        SEARCHBUFFER * Downsample, 
                                        n - SEARCHBUFFER * Downsample + DATAPADDING_MSECS  * (Fs / 1000),
                                        maxNsamples - 2 * SEARCHBUFFER * Downsample + DATAPADDING_MSECS  * (Fs / 1000));

    global_scale = (float) sqrt (TARGET_AVG_POWER / power_above_300Hz); 

    for (i = 0; i < n; i++) {
        info-> data [i] *= global_scale;    
    }

    safe_free (align_filtered);
}

void pesq_raw 
(
    SIGNAL_INFO * ref_info, 
    SIGNAL_INFO * deg_info,
    ERROR_INFO * err_info, 
    long * Error_Flag, 
    char ** Error_Type
)
{
    long    WB_InIIR_Nsos = 0;
    float * WB_InIIR_Hsos = NULL;

    float * ftmp = NULL;

    // ref_info-> data = NULL;
    // ref_info-> VAD = NULL;
    // ref_info-> logVAD = NULL;
    
    // deg_info-> data = NULL;
    // deg_info-> VAD = NULL;
    // deg_info-> logVAD = NULL;
        
    // if ((*Error_Flag) == 0)
    // {
    //     printf ("Reading reference file %s...", ref_info-> path_name);

    //    load_src (Error_Flag, Error_Type, ref_info);
    //    if ((*Error_Flag) == 0)
    //        printf ("done.\n");
    // }
    // if ((*Error_Flag) == 0)
    // {
    //     printf ("Reading degraded file %s...", deg_info-> path_name);

    //    load_src (Error_Flag, Error_Type, deg_info);
    //    if ((*Error_Flag) == 0)
    //        printf ("done.\n");
    // }

    // printf (" ref_info->Nsamples=%ld \n", ref_info->Nsamples);  
    // printf (" deg_info->Nsamples=%ld \n", deg_info->Nsamples);  

    if (((ref_info-> Nsamples - 2 * SEARCHBUFFER * Downsample < Fs / 4) ||
         (deg_info-> Nsamples - 2 * SEARCHBUFFER * Downsample < Fs / 4)) &&
        ((*Error_Flag) == 0))
    {
        (*Error_Flag) = 2;
        (*Error_Type) = "Reference or Degraded below 1/4 second - processing stopped ";
    }

    if ((*Error_Flag) == 0)
    {
        alloc_other (ref_info, deg_info, Error_Flag, Error_Type, &ftmp);
    }

    if ((*Error_Flag) == 0)
    {   
        int     maxNsamples = max (ref_info-> Nsamples, deg_info-> Nsamples);
        float * model_ref; 
        float * model_deg; 
        long    i;
        // FILE *resultsFile;

        // printf (" Level normalization...\n");            
        fix_power_level (ref_info, "reference", maxNsamples);
        fix_power_level (deg_info, "degraded", maxNsamples);

        // printf (" IRS filtering...\n"); 
        if( Fs == 16000 ) {
            WB_InIIR_Nsos = WB_InIIR_Nsos_16k;
            WB_InIIR_Hsos = WB_InIIR_Hsos_16k;
        } else {
            WB_InIIR_Nsos = WB_InIIR_Nsos_8k;
            WB_InIIR_Hsos = WB_InIIR_Hsos_8k;
        }
        if( ref_info->input_filter == 1 ) {
            apply_filter (ref_info-> data, ref_info-> Nsamples, 26, standard_IRS_filter_dB);
        } else {
            for( i = 0; i < 16; i++ ) {
                ref_info->data[SEARCHBUFFER * Downsample + i - 1]
                    *= (float)i / 16.0f;
                ref_info->data[ref_info->Nsamples - SEARCHBUFFER * Downsample - i]
                    *= (float)i / 16.0f;
            }
            IIRFilt( WB_InIIR_Hsos, WB_InIIR_Nsos, NULL,
                 ref_info->data + SEARCHBUFFER * Downsample,
                 ref_info->Nsamples - 2 * SEARCHBUFFER * Downsample, NULL );
        }
        if( deg_info->input_filter == 1 ) {
            apply_filter (deg_info-> data, deg_info-> Nsamples, 26, standard_IRS_filter_dB);
        } else {
            for( i = 0; i < 16; i++ ) {
                deg_info->data[SEARCHBUFFER * Downsample + i - 1]
                    *= (float)i / 16.0f;
                deg_info->data[deg_info->Nsamples - SEARCHBUFFER * Downsample - i]
                    *= (float)i / 16.0f;
            }
            IIRFilt( WB_InIIR_Hsos, WB_InIIR_Nsos, NULL,
                 deg_info->data + SEARCHBUFFER * Downsample,
                 deg_info->Nsamples - 2 * SEARCHBUFFER * Downsample, NULL );
        }

        model_ref = (float *) safe_malloc ((ref_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000)) * sizeof (float));
        model_deg = (float *) safe_malloc ((deg_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000)) * sizeof (float));

        for (i = 0; i < ref_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000); i++) {
            model_ref [i] = ref_info-> data [i];
        }
    
        for (i = 0; i < deg_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000); i++) {
            model_deg [i] = deg_info-> data [i];
        }
    
        input_filter( ref_info, deg_info, ftmp );

        // printf (" Variable delay compensation...\n");            
        calc_VAD (ref_info);
        calc_VAD (deg_info);
        
        crude_align (ref_info, deg_info, err_info, WHOLE_SIGNAL, ftmp);

        utterance_locate (ref_info, deg_info, err_info, ftmp);
    
        for (i = 0; i < ref_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000); i++) {
            ref_info-> data [i] = model_ref [i];
        }
    
        for (i = 0; i < deg_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000); i++) {
            deg_info-> data [i] = model_deg [i];
        }

        safe_free (model_ref);
        safe_free (model_deg); 
    
        if ((*Error_Flag) == 0) {
            if (ref_info-> Nsamples < deg_info-> Nsamples) {
                float *new_ref = (float *) safe_malloc((deg_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000)) * sizeof(float));
                long  i;
                for (i = 0; i < ref_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000); i++) {
                    new_ref [i] = ref_info-> data [i];
                }
                for (i = ref_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000); 
                     i < deg_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000); i++) {
                    new_ref [i] = 0.0f;
                }
                safe_free (ref_info-> data);
                ref_info-> data = new_ref;
                new_ref = NULL;
            } else {
                if (ref_info-> Nsamples > deg_info-> Nsamples) {
                    float *new_deg = (float *) safe_malloc((ref_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000)) * sizeof(float));
                    long  i;
                    for (i = 0; i < deg_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000); i++) {
                        new_deg [i] = deg_info-> data [i];
                    }
                    for (i = deg_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000); 
                         i < ref_info-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000); i++) {
                        new_deg [i] = 0.0f;
                    }
                    safe_free (deg_info-> data);
                    deg_info-> data = new_deg;
                    new_deg = NULL;
                }
            }
        }        

        // printf (" Acoustic model processing...\n");    
        pesq_psychoacoustic_model (ref_info, deg_info, err_info, ftmp);
    
        // safe_free (ref_info-> data);
        // safe_free (ref_info-> VAD);
        // safe_free (ref_info-> logVAD);
        // safe_free (deg_info-> data);
        // safe_free (deg_info-> VAD);
        // safe_free (deg_info-> logVAD);

        safe_free (ftmp);

		if ( err_info->mode == NB_MODE )
		{
			err_info->mapped_mos = 0.999f+4.0f/(1.0f+(float)exp((-1.4945f*err_info->pesq_mos+4.6607f)));
		}
		else
		{
			err_info->mapped_mos = 0.999f+4.0f/(1.0f+(float)exp((-1.3669f*err_info->pesq_mos+3.8224f)));
			err_info->pesq_mos = -1.0;
		}

        // resultsFile = fopen (ITU_RESULTS_FILE, "at");

        // if (resultsFile != NULL) {
        //     long start, end;

        //     if (0 != fseek (resultsFile, 0, SEEK_SET)) {
        //         printf ("Could not move to start of results file %s!\n", ITU_RESULTS_FILE);
        //         exit (1);
        //     }
        //     start = ftell (resultsFile);

        //     if (0 != fseek (resultsFile, 0, SEEK_END)) {
        //         printf ("Could not move to end of results file %s!\n", ITU_RESULTS_FILE);
        //         exit (1);
        //     }
        //     end = ftell (resultsFile);

        //     if (start == end) {
        //         fprintf (resultsFile, "REFERENCE\t DEGRADED\t PESQMOS\t MOSLQO\t SAMPLE_FREQ\t MODE\n"); 
		// 		fflush (resultsFile);
        //     }

        //     fprintf (resultsFile, "%s\t ", ref_info-> path_name);
        //     fprintf (resultsFile, "%s\t ", deg_info-> path_name);

		// 	fprintf (resultsFile, "%.3f\t ", err_info->pesq_mos);
        //     fprintf (resultsFile, "%.3f\t ", err_info->mapped_mos);
        //     fprintf (resultsFile, "%d\t", Fs);

		// 	if ( err_info->mode == NB_MODE )
		// 		fprintf (resultsFile, "nb");
		// 	else
		// 		fprintf (resultsFile, "wb");

        //    fprintf (resultsFile, "\n", Fs);

        //    fclose (resultsFile);
        // }

    }

    return;
}

static void fill_signal_info
(
    const short * samples_buf, 
    long sampels_len,
    short mode,
    SIGNAL_INFO * sinfo
)
{
    if (mode == NB_MODE)
    {
        sinfo->input_filter = 1;
    }
    else
    {
        sinfo->input_filter = 2;
    }
    
    sinfo->apply_swap = 0;
    sinfo-> data = NULL;
    sinfo-> VAD = NULL;
    sinfo-> logVAD = NULL;

    sinfo-> Nsamples = sampels_len + 2 * SEARCHBUFFER * Downsample;
    sinfo->data = (float *) safe_malloc( (sinfo-> Nsamples + DATAPADDING_MSECS  * (Fs / 1000)) * sizeof(float) );

    // printf("src.Nsamples=%ld, dst.Nsamples=%ld, Fs=%ld\n", sampels_len, sinfo-> Nsamples, Fs);

    float * read_ptr = sinfo-> data;
    long ci = 0;
    for( ci = SEARCHBUFFER*Downsample; ci > 0; ci-- )
    {
        *(read_ptr++) = 0.0f;
    }

    for(ci = 0; ci < sampels_len; ci++)
    {
        *(read_ptr++) = (float)(samples_buf[ci]);
    }
      
    for( ci = DATAPADDING_MSECS  * (Fs / 1000) + SEARCHBUFFER * Downsample;
         ci > 0; ci-- )
    {
        *(read_ptr++) = 0.0f;
    }
    
    sinfo-> VAD = safe_malloc( sinfo-> Nsamples * sizeof(float) / Downsample );
    sinfo-> logVAD = safe_malloc( sinfo-> Nsamples * sizeof(float) / Downsample );
}

static void free_signal_info
(
    SIGNAL_INFO * sinfo
)
{
    if (sinfo->data)
    {
        safe_free(sinfo->data);
        sinfo->data = NULL;
    }

    if (sinfo->VAD)
    {
        safe_free(sinfo->VAD);
        sinfo->VAD = NULL;
    }

    if (sinfo->logVAD)
    {
        safe_free(sinfo->logVAD);
        sinfo->logVAD = NULL;
    }
}

void pesq_measure_pcm
(
    const short * ref_samples_buf, 
    long ref_sampels_len,
    
    const short * deg_samples_buf, 
    long deg_sampels_len,
  
    long sample_rate,
    short mode,

    float *p_pesq_mos,
    float *p_mapped_mos,

    long * p_error_Flag, 
    char ** p_error_Type
)
{
    SIGNAL_INFO ref_info;
    SIGNAL_INFO deg_info;
    ERROR_INFO err_info;

    do
    {
        // short mode = NB_MODE;

        memset(&ref_info, 0, sizeof(ref_info));
        memset(&deg_info, 0, sizeof(deg_info));
        memset(&err_info, 0, sizeof(err_info));

        err_info.mode = mode;
        
        select_rate (sample_rate, p_error_Flag, p_error_Type);
        if (*p_error_Flag != 0)
        {
            break;
        }
        

        fill_signal_info(ref_samples_buf, ref_sampels_len, mode, &ref_info);
        fill_signal_info(deg_samples_buf, deg_sampels_len, mode, &deg_info);

        pesq_raw 
        (
            &ref_info, 
            &deg_info,
            &err_info, 
            p_error_Flag, 
            p_error_Type
        );

        if (*p_error_Flag != 0)
        {
            break;
        }

        *p_pesq_mos = err_info.pesq_mos;
        *p_mapped_mos = err_info.mapped_mos;

    } while (0);
    
    free_signal_info(&ref_info);
    free_signal_info(&deg_info);
}



#define ITU_RESULTS_FILE          "pesq_results.txt"

void pesq_measure_test 
(
    SIGNAL_INFO * ref_info, 
    SIGNAL_INFO * deg_info,
    ERROR_INFO * err_info, 
    long * Error_Flag, 
    char ** Error_Type
)
{
    ref_info-> data = NULL;
    ref_info-> VAD = NULL;
    ref_info-> logVAD = NULL;
    
    deg_info-> data = NULL;
    deg_info-> VAD = NULL;
    deg_info-> logVAD = NULL;
        
    if ((*Error_Flag) == 0)
    {
        printf ("Reading reference file %s...", ref_info-> path_name);

       load_src (Error_Flag, Error_Type, ref_info);
       if ((*Error_Flag) == 0)
           printf ("done.\n");
    }
    if ((*Error_Flag) == 0)
    {
        printf ("Reading degraded file %s...", deg_info-> path_name);

       load_src (Error_Flag, Error_Type, deg_info);
       if ((*Error_Flag) == 0)
           printf ("done.\n");
    }

    pesq_raw 
    (
        ref_info, 
        deg_info,
        err_info, 
        Error_Flag, 
        Error_Type
    );

    if ((*Error_Flag) == 0) 
    {
        FILE *resultsFile = fopen (ITU_RESULTS_FILE, "at");

        if (resultsFile != NULL) {
            long start, end;

            if (0 != fseek (resultsFile, 0, SEEK_SET)) {
                printf ("Could not move to start of results file %s!\n", ITU_RESULTS_FILE);
                exit (1);
            }
            start = ftell (resultsFile);

            if (0 != fseek (resultsFile, 0, SEEK_END)) {
                printf ("Could not move to end of results file %s!\n", ITU_RESULTS_FILE);
                exit (1);
            }
            end = ftell (resultsFile);

            if (start == end) {
                fprintf (resultsFile, "REFERENCE\t DEGRADED\t PESQMOS\t MOSLQO\t SAMPLE_FREQ\t MODE\n"); 
				fflush (resultsFile);
            }

            fprintf (resultsFile, "%s\t ", ref_info-> path_name);
            fprintf (resultsFile, "%s\t ", deg_info-> path_name);

			fprintf (resultsFile, "%.3f\t ", err_info->pesq_mos);
            fprintf (resultsFile, "%.3f\t ", err_info->mapped_mos);
            fprintf (resultsFile, "%ld\t", Fs);

			if ( err_info->mode == NB_MODE )
				fprintf (resultsFile, "nb");
			else
				fprintf (resultsFile, "wb");

           fprintf (resultsFile, "\n");

           fclose (resultsFile);
        }
    }

    safe_free (ref_info-> data);
    safe_free (ref_info-> VAD);
    safe_free (ref_info-> logVAD);
    safe_free (deg_info-> data);
    safe_free (deg_info-> VAD);
    safe_free (deg_info-> logVAD);

    printf ("test done\n");
}

