#include "pesqraw.h"
#include <stdio.h>
#include <stdlib.h>

static int load_file(const char *filepath, unsigned char ** pbuf, long *psize) 
{
    int ret = 0;
    FILE * file = NULL;
    unsigned char *buf = NULL;

    do
    {
        file = fopen( filepath, "rb" );
        if (!file)
        {
            ret = -__LINE__;
            break;
        }

        if( fseek( file, 0L, SEEK_END ) != 0 )
        {
            ret = -__LINE__;
            break;
        }

        long file_size = ftell( file );
        if( file_size < 0L )
        {
            ret = -__LINE__;
            break;
        }

        if( fseek( file, 0L, SEEK_SET ) != 0 )
        {
            ret = -__LINE__;
            break;
        }

        buf = (unsigned char *) malloc(file_size);

        size_t read_count = fread( buf, 1, file_size, file );
        if( read_count < file_size )
        {
            ret = -__LINE__;
            break;
        }

        *pbuf = buf;
        *psize = file_size;

        buf = NULL;
        ret = 0;
    } while (0);
    
    if (ret)
    {
        if (file)
        {
            fclose(file);
            file = NULL;
        }

        if (buf)
        {
            free(buf);
            buf = NULL;
        }
    }

    return ret;
}

static int load_pcm_file(const char *filepath, short ** pbuf, long *psize) 
{
    unsigned char * bytes_buf = NULL;
    long bytes_len = 0;
    int ret = load_file(filepath, &bytes_buf, &bytes_len);
    if (ret == 0)
    {
        *pbuf = (short *) bytes_buf;
        *psize = bytes_len / sizeof(short);
    }
    return ret;
}

int main() 
{
    const char * ref_filepath = "/tmp/pcm/sample-16000Hz-1ch.pcm";
    const char * deg_filepath = "/tmp/pcm/dst_001_mp3_16000hz1ch.pcm";
    int samplerate = 16000;
    int channels = 1;
    short mode = NB_MODE;

    short * ref_sampels_buf = NULL;
    long ref_sampels_len = 0;

    short * deg_sampels_buf = NULL;
    long deg_sampels_len = 0;

    int ret = 0;
    do
    {
        ret = load_pcm_file(ref_filepath, &ref_sampels_buf, &ref_sampels_len);
        if (ret)
        {
            printf("failed to load ref [%s]\n", ref_filepath);
            break;
        }
        printf("loaded ref [%s], samples [%ld]\n", ref_filepath, ref_sampels_len);

        ret = load_pcm_file(deg_filepath, &deg_sampels_buf, &deg_sampels_len);
        if (ret)
        {
            printf("failed to load deg [%s]\n", deg_filepath);
            break;
        }
        printf("loaded deg [%s], samples [%ld]\n", deg_filepath, deg_sampels_len);

        float pesq_mos = 0.0f;
        float mapped_mos = 0.0f;

        long error_flag = 0; 
        char * error_type = NULL;

        pesq_measure_pcm
        (
            ref_sampels_buf, 
            ref_sampels_len,
            
            deg_sampels_buf, 
            deg_sampels_len,
        
            samplerate,
            mode,

            &pesq_mos,
            &mapped_mos,

            &error_flag, 
            &error_type
        );

        if (error_flag)
        {
            printf("failed to measure pesq [%ld]-[%s]\n", error_flag, error_type);
            break;
        }

		if ( mode == NB_MODE )
        {
            printf ("P.862 Prediction (Raw MOS, MOS-LQO):  = %.3f\t%.3f\n", (double) pesq_mos,  (double) mapped_mos);
        }
        else
        {
            printf ("P.862.2 Prediction (MOS-LQO):  = %.3f\n", (double) mapped_mos);
        }        

    } while (0);
    
    if (ref_sampels_buf)
    {
        free(ref_sampels_buf);
        ref_sampels_buf = NULL;
    }

    if (deg_sampels_buf)
    {
        free(deg_sampels_buf);
        deg_sampels_buf = NULL;
    }

    return 0;
}
